﻿Imports System.Math

Public Enum ColorStyles
    Original
    Smooth1
    Smooth2
End Enum

Public Class Form1
    Private m_DrawingBox As Boolean
    Private m_StartX, m_StartY As Double
    Private m_CurX, m_CurY As Double

    Private m_Xmin, m_Xmax, m_Ymin, m_Ymax As Double

    Public MaxIterations As Integer
    Public Z0 As Complex

    Public ColorStyle As ColorStyles = ColorStyles.Original

    Public NumColors As Integer
    Public Colors() As Color

    Private m_Bm As Bitmap

    Private Const MIN_X As Double = -2.2
    Private Const MAX_X As Double = 1
    Private Const MIN_Y As Double = -1.2
    Private Const MAX_Y As Double = 1.2

    ' Reset the number of colors to 0.
    Public Sub ResetColors()
        NumColors = 0
        Erase Colors
    End Sub

    ' Add this color to the list.
    Public Sub AddColor(ByVal new_color As Color)
        NumColors += 1
        ReDim Preserve Colors(NumColors - 1)
        Colors(NumColors - 1) = new_color
    End Sub

    ' Adjust the aspect ratio of the selected
    ' coordinates so they fit the window properly.
    Private Sub AdjustAspect()
        Dim want_aspect As Double = (m_Ymax - m_Ymin) / (m_Xmax - m_Xmin)
        Dim picCanvas_aspect As Double = _
            picCanvas.ClientSize.Height / picCanvas.ClientSize.Width
        Dim wid, hgt, mid As Double
        If want_aspect > picCanvas_aspect Then
            ' The selected area is too tall and thin.
            ' Make it wider.
            wid = (m_Ymax - m_Ymin) / picCanvas_aspect
            mid = (m_Xmin + m_Xmax) / 2
            m_Xmin = mid - wid / 2
            m_Xmax = mid + wid / 2
        Else
            ' The selected area is too short and wide.
            ' Make it taller.
            hgt = (m_Xmax - m_Xmin) * picCanvas_aspect
            mid = (m_Ymin + m_Ymax) / 2
            m_Ymin = mid - hgt / 2
            m_Ymax = mid + hgt / 2
        End If
    End Sub

    ' Draw the Mandelbrot set.
    Private Sub DrawMandelbrot()
        ' Work until |Zn| >= escape_radius
        Const escape_radius As Double = 2.0

        ' Work until the |Zn|^2 >= escape_radius^2.
        Dim escape_squared As Double = escape_radius * escape_radius

        ' Used for color calculations.
        Dim log_escape As Double = Math.Log(escape_radius)

        ' Make a Bitmap to draw on.
        m_Bm = New Bitmap(picCanvas.ClientSize.Width, picCanvas.ClientSize.Height)
        Using gr As Graphics = Graphics.FromImage(m_Bm)
            ' Clear.
            gr.Clear(picCanvas.BackColor)
            picCanvas.Image = m_Bm
            Application.DoEvents()

            ' Adjust the coordinate bounds to fit picCanvas.
            AdjustAspect()

            ' dReaC is the change in the real part
            ' (X value) for C. dImaC is the change in the
            ' imaginary part (Y value).
            Dim wid As Integer = picCanvas.ClientRectangle.Width
            Dim hgt As Integer = picCanvas.ClientRectangle.Height
            Dim dReaC As Double = (m_Xmax - m_Xmin) / (wid - 1)
            Dim dImaC As Double = (m_Ymax - m_Ymin) / (hgt - 1)

            ' Calculate the values.
            Dim ReaC As Double = m_Xmin
            For X = 0 To wid - 1
                Dim ImaC As Double = m_Ymin
                For Y = 0 To hgt - 1
                    Dim Z As Complex = Z0
                    Dim C As New Complex(ReaC, ImaC)
                    Dim iteration As Integer = 1
                    Do While iteration < MaxIterations And Z.MagnitudeSquared < escape_squared
                        ' Calculate Z(clr).
                        Z = Z * Z + C
                        iteration = iteration + 1
                    Loop

                    ' Set the pixel's value.
                    If (iteration >= MaxIterations) Then
                        m_Bm.SetPixel(X, Y, Color.Black)
                    Else
                        If (ColorStyle = ColorStyles.Original) Then
                            m_Bm.SetPixel(X, Y, Colors(iteration Mod Colors.Count))
                        Else
                            ' Reduce the error in mu.
                            For i As Integer = 0 To 2
                                Z = Z * Z + C
                                iteration += 1
                            Next i

                            Dim mu As Double = iteration + 1 - _
                                Math.Log(Math.Log(Z.Magnitude)) / log_escape
                            If (ColorStyle = ColorStyles.Smooth2) Then
                                mu = mu / MaxIterations * Colors.Count
                            End If
                            m_Bm.SetPixel(X, Y, GetColor(mu))
                        End If
                    End If

                    ImaC = ImaC + dImaC
                Next Y
                ReaC = ReaC + dReaC

                ' Let the user know we're not dead.
                If X Mod 10 = 0 Then
                    picCanvas.Refresh()
                End If
            Next X
        End Using

        Text = "Mandelbrot (" & _
            m_Xmin.ToString("0.000000") & ", " & _
            m_Ymin.ToString("0.000000") & ")-(" & _
            m_Xmax.ToString("0.000000") & ", " & _
            m_Ymax.ToString("0.000000") & ")"
    End Sub

    ' Scale the selected area by this factor.
    Private Sub ScaleArea(ByVal scale_factor As Integer)
        Dim size As Double = scale_factor * (m_Xmax - m_Xmin)
        If size > 3.2 Then
            mnuScaleFull_Click(Nothing, Nothing)
            Exit Sub
        End If
        Dim mid As Double = (m_Xmin + m_Xmax) / 2
        m_Xmin = mid - size / 2
        m_Xmax = mid + size / 2

        size = scale_factor * (m_Ymax - m_Ymin)
        If size > 2.4 Then
            mnuScaleFull_Click(Nothing, Nothing)
            Exit Sub
        End If
        mid = (m_Ymin + m_Ymax) / 2
        m_Ymin = mid - size / 2
        m_Ymax = mid + size / 2

        Me.Cursor = Cursors.WaitCursor
        Application.DoEvents()
        DrawMandelbrot()
        Me.Cursor = Cursors.Default
        picCanvas.Cursor = Cursors.Cross
    End Sub

    Private Sub mnuScale_2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuScale_2.Click
        ScaleArea(2)
    End Sub
    Private Sub mnuScale_4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuScale_4.Click
        ScaleArea(4)
    End Sub
    Private Sub mnuScale_8_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuScale_8.Click
        ScaleArea(8)
    End Sub
    ' Zoom out to full scale.
    Private Sub mnuScaleFull_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuScaleFull.Click
        m_Xmin = MIN_X
        m_Xmax = MAX_X
        m_Ymin = MIN_Y
        m_Ymax = MAX_Y

        Me.Cursor = Cursors.WaitCursor
        Application.DoEvents()
        DrawMandelbrot()
        Me.Cursor = Cursors.Default
        picCanvas.Cursor = Cursors.Cross
    End Sub

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Me.Show()
        Application.DoEvents()

        MaxIterations = 64
        Z0 = New Complex(0, 0)

        ' Create some default colors.
        ResetColors()
        Dim frm As New MandelbrotConfig()
        frm.Initialize(Me)
        frm.SetDefaultColors()
        frm.AcceptSelections()
        frm.Close()

        ' Display the first Mandelbrot set.
        mnuScaleFull_Click(Nothing, Nothing)
    End Sub

    Private Sub mnuOptOptions_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuOptOptions.Click
        Dim frm As New MandelbrotConfig()
        frm.Initialize(Me)
        frm.ShowDialog()
    End Sub

    ' Refresh. This is useful when the user resizes the form.
    Private Sub mnuScaleRefresh_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuScaleRefresh.Click
        ScaleArea(1)
    End Sub

    Private Sub picCanvas_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseDown
        m_DrawingBox = True
        m_StartX = e.X
        m_StartY = e.Y
        m_CurX = e.X
        m_CurY = e.Y
    End Sub

    Private Sub picCanvas_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseMove
        If Not m_DrawingBox Then Exit Sub

        m_CurX = e.X
        m_CurY = e.Y

        Dim bm As New Bitmap(m_Bm)
        Using gr As Graphics = Graphics.FromImage(bm)
            gr.DrawRectangle(Pens.Yellow, _
                CInt(Min(m_StartX, m_CurX)), CInt(Min(m_StartY, m_CurY)), _
                CInt(Abs(m_StartX - m_CurX)), CInt(Abs(m_StartY - m_CurY)))
        End Using
        picCanvas.Image = bm
    End Sub

    Private Sub picCanvas_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseUp
        If Not m_DrawingBox Then Exit Sub
        m_DrawingBox = False
        picCanvas.Image = m_Bm

        m_CurX = e.X
        m_CurY = e.Y

        ' Put the coordinates in proper order.
        Dim x1 As Double = Min(m_StartX, m_CurX)
        Dim x2 As Double = Max(m_StartX, m_CurX)
        If x1 = x2 Then x2 = x1 + 1

        Dim y1 As Double = Min(m_StartY, m_CurY)
        Dim y2 As Double = Max(m_StartY, m_CurY)
        If y1 = y2 Then y2 = y1 + 1

        ' Convert screen coords into drawing coords.
        Dim factor As Double = (m_Xmax - m_Xmin) / picCanvas.ClientSize.Width
        m_Xmax = m_Xmin + x2 * factor
        m_Xmin = m_Xmin + x1 * factor

        factor = (m_Ymax - m_Ymin) / picCanvas.ClientSize.Height
        m_Ymax = m_Ymin + y2 * factor
        m_Ymin = m_Ymin + y1 * factor

        Me.Cursor = Cursors.WaitCursor
        Application.DoEvents()
        DrawMandelbrot()
        Me.Cursor = Cursors.Default
        picCanvas.Cursor = Cursors.Cross
    End Sub

    ' Get a color for this pixel.
    Private Function GetColor(ByVal mu As Double) As Color
        Dim clr1 As Integer = CInt(Int(mu))
        Dim t2 As Double = mu - clr1
        Dim t1 As Double = 1 - t2
        clr1 = clr1 Mod Colors.Count
        Dim clr2 As Integer = (clr1 + 1) Mod Colors.Count

        Dim r As Byte = CByte(Colors(clr1).R * t1 + Colors(clr2).R * t2)
        Dim g As Byte = CByte(Colors(clr1).G * t1 + Colors(clr2).G * t2)
        Dim b As Byte = CByte(Colors(clr1).B * t1 + Colors(clr2).B * t2)

        Return Color.FromArgb(255, r, g, b)
    End Function
End Class
